import { replaceHtmlText } from "@/directive/highlight";
import { urlPattern2 } from "@/utils/validate";
import { getOpenHref } from "@/utils/tools";
import { ref } from 'vue'
import axios from 'axios'

/**
 * @description: 处理html字符串中的img标签
 * @param {string} html
 * @return {*}
 */
export function formatImgText(html: string) {
  if (!html) return "";
  const replacedHtmlStr = html.replace(/<img.*?>/g, "[图片]");
  const dom: any = document
    .createRange()
    .createContextualFragment("<div></div>");
  dom.firstChild.innerHTML = replacedHtmlStr;
  return dom.firstChild.textContent;
}

/**
 * @description: 格式化文件大小
 * @param {number} size
 * @return {*}
 */
export function formatFileSize(size: number) {
  if (size < 1024) {
    return size + " B";
  } else if (size < 1024 * 1024) {
    return (size / 1024).toFixed(2) + " KB";
  } else if (size < 1024 * 1024 * 1024) {
    return (size / (1024 * 1024)).toFixed(2) + " MB";
  } else {
    return (size / (1024 * 1024 * 1024)).toFixed(2) + " GB";
  }
}

export const downloadFileList = ref([])
export function removeDownloadFile(message_id) {
  const index = downloadFileList.value.findIndex((item) => item.message_id === message_id)
  if (index !== -1) {
    downloadFileList.value.splice(index, 1)
  }
}
export const addDownloadFile = (message_id) => {
  const index = downloadFileList.value.findIndex((item) => item.message_id === message_id)
  if (index === -1) {
    downloadFileList.value.push({ message_id, percent: '0%' })
  }
}
export const updateFileProcess = (message_id, percent) => {
  const index = downloadFileList.value.findIndex((item) => item.message_id === message_id)
  if (index !== -1) {
    downloadFileList.value[index] = { message_id, percent }
  }
}

/**
 * @description: 下载图片文件
 * @param {number} size
 * @return {*}
 */
export function downloadImg(url: string, filename: string) {
  const xhr = new XMLHttpRequest();
  xhr.open("GET", url, true);
  xhr.responseType = "blob";
  xhr.onload = function () {
    if (xhr.status === 200) {
      const blob = xhr.response;
      const a = document.createElement("a");
      const url = URL.createObjectURL(blob);
      a.href = url;
      a.download = filename;
      document.body.appendChild(a);
      a.click();
      document.body.removeChild(a);
      URL.revokeObjectURL(url);
    }
  };
  xhr.send();
}

/**
 * @description: 下载其他文件
 * @param {string} url
 * @param {string} filename
 * @return {*}
 */
export const downloadFile = async ({ url, name }, message_id) => {
  if (message_id) {
    addDownloadFile(message_id)
  }
  return axios
    .get(`${url}?response-content-type=application/octet-stream`, {
      responseType: 'blob',
      onDownloadProgress(progressEvent) {
        if (message_id) {
          updateFileProcess(message_id, (progressEvent.progress * 100).toFixed(0) + '%')
        }
      }
    })
    .then((res) => {
      const objectUrl = URL.createObjectURL(res.data)
      const a = document.createElement('a')
      a.href = objectUrl
      a.download = name
      a.click()
      window.URL.revokeObjectURL(res.data)
    })
    .finally(() => {
      if (message_id) {
        removeDownloadFile(message_id)
      }
    })
}
// export function downloadFile(url: string, filename: string) {
//   const x = new window.XMLHttpRequest();
//   x.open("GET", url, true);
//   x.responseType = "blob";
//   x.onload = () => {
//     const newUrl = window.URL.createObjectURL(x.response);
//     const a = document.createElement("a");
//     a.href = newUrl;
//     a.download = filename;
//     a.click();
//   };
//   x.send();
// }

/**
 * @description: 判断字符串是否能被JSON.parse解析
 * @param {string} str
 * @return {*}
 */
export function isJsonString(str: string) {
  try {
    const obj = JSON.parse(str);
    return typeof obj === "object" && obj !== null && !Array.isArray(obj);
  } catch (error) {
    return false;
  }
}

/**
 * @description: 判断时间是否超过指定的分钟数
 * @param {string} time ：输入的时间
 * @param {number} minute：设置超过分钟数
 * @return {*}
 */
export function isTimeExceeded(time: string, minute: number) {
  const givenTime = new Date(time).getTime();
  const currentTime = new Date().getTime();
  const difference = currentTime - givenTime;
  const twoMinutes = minute * 60 * 1000;
  if (difference > twoMinutes) {
    return true;
  } else {
    return false;
  }
}

/**
 * @description: 格式化文本内容
 * @param {string} msg
 * @return {*}
 */
export function htmlTextFormat(msg: string) {
  return replaceHtmlText(msg, urlPattern2, (url: string) => {
    return `<a href="${getOpenHref(
      url
    )}" target="_blank" class="chat_url">${url}</a>`;
  });
}
