import { reactive } from 'vue';
import { SOCKET, ENV } from '@/api/baseApi';
import { useUserImStore } from '@/store/userIm';
import bus from '@/utils/mitt';
import throttle from 'lodash/throttle';
import { closeReConnectTimer, reConnectTimer } from './reConnectTime';
import {
  reSendAfterSocketSuccess,
  updateTempMessage,
  setReSendState,
} from '@/components/chat-module/db/tempMessage';
import { addPendingMessage } from '@/components/chat-module/msgUtils/tempMessageAction';
import dayjs from 'dayjs';
import { messageHandle } from './messageHandle.js';
import Cookies from 'js-cookie'

const systemCmd = ['connect_success', 'init_success'];
const appVersion = '1.0.0';
const platform = 'pc';
const source = 'see';
let imStore = null;
const isSystemSleep = false;
const log = (type: string, name: string, content?: any) => {
  console.log(type, new Date().toLocaleTimeString(), name, content || '');
};
export class WebSocketConnection {
  private connectType: string;
  private url: string;
  private socket: WebSocket | null;
  private im_id: string;
  /* 注册事件 */
  private socketEvent: any;
  /* 是否正在重连 */
  private isReConnecting: boolean;
  private timer: any;
  /* 设置发送心跳时间 */
  private getHeartbeatInterval: number;
  private pingStack: any;

  constructor(connectType: string, url) {
    this.connectType = connectType;
    this.url = SOCKET;
    this.im_id = '';
    this.socket = null;
    this.isReConnecting = false;
    this.socketEvent = null;
    this.timer = null;
    this.getHeartbeatInterval = 30 * 1000;
    this.pingStack = {
      list: [],
      clear() {
        this.list = [];
      },
      timer: '',
      push(self: any) {
        this.list.push({ cmd: 'ping' });
        this.timer = setTimeout(() => {
          if (!self.isReConnecting && this.length() !== 0) {
            self.tryReConnect();
          }
        }, 5000);
      },
      pop() {
        this.list.pop();
        clearTimeout(this.timer);
      },
      length() {
        return this.list.length;
      },
    };
  }
  /* 连接websocket */
  connect(): void {
    imStore = imStore || useUserImStore();
    this.im_id = imStore.user_im_id;
    if (this.socket || !navigator.onLine || isSystemSleep) return;
    this.socket = new WebSocket(
      SOCKET + `?uniq_id=${this.im_id}&version=${appVersion}&platform=${platform}&source=${imStore.source || source}&token=${Cookies.get(`lsn-token${ENV === 'dev' ? '-dev' : ''}`)}`,
    );
    const socketCreateTime = dayjs().format('YYYY-MM-DD HH:mm:ss');

    this.socket.onopen = () => {};

    this.socket.onclose = e => {
      clearTimeout(this.timer);
      this.socket = null;
      setReSendState();
      setTimeout(() => {
        this.tryReConnect();
      }, 1000);
    };
    this.socket.onerror = e => {
      setTimeout(() => {
        this.tryReConnect();
      }, 1000);
    };

    this.socket.onmessage = ({ data }) => {
      if (typeof data === 'string') {
        const msg = JSON.parse(data);
        const body = msg.body;
        switch (msg.cmd) {
          // 连接成功
          case 'connect_success':
            if (this.im_id) {
              const init = { cmd: 'init', body: { uniq_id: this.im_id } };
              this.socketSend(init);
              log('socketStatus', 'socket_connect_success', this.connectType);
            }
            // imStore.GET_NAV_LIST_MP().then((im_ids) => {
            //   this.getUnReadTotal()
            //   this.socketSend({ cmd: 'bindChild', body: { im_ids } })
            // })
            // imStore.GET_NAV_LIST().then((im_ids) => {
            //   if (im_ids.length) {
            //     imStore.GET_READ_NUM()
            //     this.socketSend({ cmd: 'bindChild', body: { im_ids } })
            //   }
            // })
            break;
          // 初始化成功
          case 'init_success':
            this.getUnReadTotal();
            this.heartbeat();
            closeReConnectTimer();
            this.isReConnecting = false;
            reSendAfterSocketSuccess(socketCreateTime);
            log('socketStatus', 'socket_init_success');
            break;
          // 初始化失败
          case 'init_fail':
            this.tryReConnect();
            log('socketStatus', 'socket_init_fail');
            break;
          case 'pong':
            this.pingStack.pop();
            break;
          // 发送失败
          case 'send_fail':
            reSendAfterSocketSuccess(socketCreateTime);
            log('socketStatus', 'socket_send_fail');
            break;
          // 消息
          case 'message':
            messageHandle(body, this.connectType, this.im_id);
            break;
        }
        if (this.socketEvent && body) {
          this.socketEvent(body);
        }
      }
    };
  }
  private sendHeartbeatData(): void {
    if (this.socket && this.socket.readyState === 1) {
      this.socketSend({ cmd: 'ping' });
      this.pingStack.push(this);
    }
  }
  private heartbeat(): void {
    // 心跳
    // 客户端和服务端沉寂一段时间后，socket会自动断开
    clearTimeout(this.timer);
    this.timer = setTimeout(() => {
      this.sendHeartbeatData();
      this.heartbeat();
    }, this.getHeartbeatInterval);
  }
  /**
   * @description: 重连轮询
   * @return {*}
   */
  private reConnectSocket(isInit?: boolean): void {
    this.isReConnecting = true;
    this.disconnect();
    setReSendState();
    this.connect();
    reConnectTimer(() => {
      if (this.isReConnecting) {
        this.reConnectSocket();
      }
    }, isInit);
  }
  /**
   * @description: 尝试重连，如果此时正在重连则不执行
   * @return {*}
   */
  tryReConnect(): void {
    log('enter_tryReConnect', '', this.isReConnecting);
    if (!this.isReConnecting) {
      log('socketStatus', 'socket_tryReConnect');
      this.reConnectSocket(true);
    }
  }
  /**
   * @description: 开始重连
   * @return {*}
   */
  startReConnect(): void {
    log('socketStatus', 'socket_startReConnect');
    throttle(
      () => {
        bus.emit('SOCKET_RELOAD');
        closeReConnectTimer();
        this.isReConnecting = false;
        this.tryReConnect();
      },
      2000,
      { leading: true },
    )();
  }
  /**
   * @description: 初始化获取未读消息条数
   */
  private getUnReadTotal(): void {}
  /**
   * @description: 发送socket事件
   */
  socketSend(payload: any): void {
    const data = JSON.stringify(payload);
    if (this.socket && this.socket.readyState === 1) {
      this.socket.send(data);
    } else {
      if (!this.socket) {
        this.tryReConnect();
      }
    }
    // 消息发送单独处理
    const custom_id = payload.body?.custom_id;
    custom_id && addPendingMessage(custom_id, this.connectType);
    if (!this.socket) {
      custom_id && updateTempMessage(custom_id, { isAutoResend: true });
    }
  }
  /* 自定义执行事件 */
  onSocketEvent(func: any): void {
    if (!func) return;
    this.socketEvent = func;
  }
  /* 移出监听事件 */
  offSocketEvent(): void {
    this.socketEvent = null;
  }
  /**
   * @description: 关闭websocket连接
   * @return {*}
   */
  disconnect(): void {
    try {
      if (this.socket) {
        this.socket.onopen = () => void 0;
        this.socket.onmessage = () => void 0;
        this.socket.onerror = () => void 0;
        this.socket.onclose = () => void 0;
        this.socket.close();
        this.socket = null;
        log('socketStatus', 'socket_close');
      }
    } catch (error) {
      log('socketStatus', 'socket_error', error);
    }
  }
}

export const CONNECT_LINK_SOCKET: WebSocketConnection = new WebSocketConnection('see');

export function socketSendFn(payload: any, connectType?: string) {
  CONNECT_LINK_SOCKET.socketSend(payload);
}
export function tryReConnectFn(connectType: string) {
  CONNECT_LINK_SOCKET.tryReConnect();
}
