/* eslint-disable */

/*
 使用的时候一定要注意及时调用clear方法，以免造成内存泄露
 推荐在面板隐藏时和组件销毁时都调用clear, 最好每次new时把之前创建的clear掉
*/
import { getDOM, getPaddingAndBorderSize, isAncestor } from '@/utils/dom'
import { nextTick } from 'vue'

export class KeyboardSelect {
  private itemElClassName: string
  private disabledClassName: string
  private selectClassName: string
  private enterFn: any
  private scrollEl: any
  private topOffset: string | number
  private bottomOffset: string | number
  private disabled: boolean
  private onSelectChange: any
  private selectIndex: any

  constructor({
    scrollEl,
    itemElClassName = 'data-item',
    disabledClassName,
    selectIndex = -1,
    selectClassName = 'is-hover',
    topOffset,
    bottomOffset,
    handleHover = false,
    enterFn, // 按回车时的回调
    onSelectChange // hover改变时触发
  }: any) {
    this.itemElClassName = itemElClassName
    this.disabledClassName = disabledClassName
    this.selectClassName = selectClassName
    this.enterFn = enterFn
    this.scrollEl = getDOM(scrollEl)
    this.topOffset = topOffset
    this.bottomOffset = bottomOffset
    this.disabled = false
    this.onSelectChange = onSelectChange
    this.selectIndex = selectIndex

    nextTick(() => {
      this.init(selectIndex)
      if (handleHover) {
        this.scrollEl?.addEventListener('mouseover', this.onMouseover)
      }
      document.addEventListener('keydown', this.onKeydown)
    })
  }
  init(selectIndex:any = -1) {
    nextTick(() => {
      const itemEls = this.getEls()
      const length = itemEls.length
      if (this.topOffset === void 0 || this.bottomOffset === void 0) {
        const { pt, pb } = getPaddingAndBorderSize(this.scrollEl)
        if (this.topOffset === void 0) {
          this.topOffset = pt
        }
        if (this.bottomOffset === void 0) {
          this.bottomOffset = pb
        }
      }
      let defaultSelectIndex = typeof selectIndex === 'function' ? selectIndex(itemEls) : selectIndex
      if (!length) {
        defaultSelectIndex = -1
      } else if (length === 1) {
        defaultSelectIndex = 0
      }
      this.setSelectIndex(defaultSelectIndex)
    })
  }
  getEls() {
    return [...this.scrollEl?.getElementsByClassName(this.itemElClassName)]
  }
  isAllDisabled() {
    return this.getEls().every(el => el.classList.contains(this.disabledClassName))
  }
  getSelectIndex(isDown: boolean) {
    let { selectIndex } = this
    const itemEls = this.getEls()
    const length = itemEls.length
    if (this.isAllDisabled() || !length) return -1
    if (isDown) {
      do {
        if (selectIndex < length - 1) {
          selectIndex += 1
        } else {
          selectIndex = 0
        }
      } while (itemEls[selectIndex].classList.contains(this.disabledClassName))
    } else {
      do {
        if (selectIndex > 0) {
          selectIndex -= 1
        } else {
          selectIndex = length - 1
        }
      } while (selectIndex > -1 && itemEls[selectIndex].classList.contains(this.disabledClassName))
    }
    return selectIndex
  }
  setSelectIndex(selectIndex: number) {
    // console.log('setSelectIndex', selectIndex)
    this.getEls().forEach((el, index) => {
      el.classList[selectIndex === index ? 'add' : 'remove'](this.selectClassName)
    })
    if (this.selectIndex !== selectIndex) {
      this.onSelectChange && this.onSelectChange(selectIndex)
      this.selectIndex = selectIndex
    }
  }
  judgeSelectOptionPlacement(selectIndex: number) { // 判断键盘选中option处于什么位置（视图上方，视图中间，视图下方）
    if (selectIndex === -1) return
    const itemEls = this.getEls()
    const { offsetTop } = itemEls[selectIndex]
    const { scrollTop, clientHeight } = this.scrollEl
    const { topOffset }: any = this
    if (scrollTop > offsetTop - topOffset) {
      return 'top'
    } else if (scrollTop < offsetTop - clientHeight + itemEls[selectIndex].offsetHeight + this.bottomOffset) {
      return 'bottom'
    } else {
      return 'center'
    }
  }
  onKeydown = (ev: any) => {
    if (this.disabled) return
    const { key } = ev
    const itemEls = this.getEls()
    if (key === 'ArrowDown' || key === 'ArrowUp') {
      const { scrollEl: el, topOffset, bottomOffset }: any = this
      const selectIndex = this.getSelectIndex(key === 'ArrowDown')
      this.setSelectIndex(selectIndex)
      if (selectIndex === -1) return
      const placement = this.judgeSelectOptionPlacement(selectIndex)
      if (placement === 'top') {
        el.scrollTop = itemEls[selectIndex].offsetTop - topOffset
      } else if (placement === 'bottom') {
        el.scrollTop = itemEls[selectIndex].offsetTop - el.clientHeight + itemEls[selectIndex].offsetHeight + bottomOffset
      }
      ev.preventDefault && ev.preventDefault()
    } else if (key === 'Enter') {
      if (this.selectIndex > -1 && itemEls.length && this.enterFn) {
        this.enterFn(itemEls[this.selectIndex], this.selectIndex)
      }
    }
  }
  onMouseover = (e: any) => {
    const el = isAncestor(e.target, this.itemElClassName)
    if (el) {
      const index = this.getEls().findIndex(e => e === el)
      this.setSelectIndex(index)
    }
  }
  clear() {
    document.removeEventListener('keydown', this.onKeydown)
    this.scrollEl.removeEventListener('mouseover', this.onMouseover)
  }
}

