import { ref, nextTick } from 'vue';
import { API_getFoldRoomList, API_createRoom, API_getUserInfo } from '@/api/im/index';
import { useUserImStore } from '@/store/userIm';
import { ElMessage } from 'element-plus';
import dayjs from 'dayjs';

function getChatSortTime(a) {
  return a.last_msg_info?.id || 0;
}
function chatListSort(arr: any) {
  let topArr = [];
  let otherArr = [];
  arr.forEach(item => {
    if (item.topping) {
      topArr.push(item);
    } else {
      otherArr.push(item);
    }
  });
  topArr.sort((a, b) => {
    const timeA = getChatSortTime(a);
    const timeB = getChatSortTime(b);
    if (timeA === timeB) {
      return 0;
    }
    return timeB > timeA ? 1 : -1;
  });
  otherArr.sort((a, b) => {
    const timeA = getChatSortTime(a);
    const timeB = getChatSortTime(b);
    if (timeA === timeB) {
      return 0;
    }
    return timeB > timeA ? 1 : -1;
  });
  return topArr.concat(otherArr);
}

function formatRoomTime(otherParams: any) {
  if (!Object.keys(otherParams).length) return {}
  // 允许只传开始时间
  let { room_create_time_start, room_create_time_end } = otherParams
  if (!room_create_time_start) return {}
  if (!room_create_time_end) room_create_time_end = dayjs().format('YYYY-MM-DD HH:mm:ss')
  return { room_create_time_start, room_create_time_end }
}

export const useRoomList = () => {
  const imStore = useUserImStore();
  const roomList = ref<any>([]);
  const allRoomList = ref<any>([]);
  const chatLoading = ref<boolean>(false);

  /**
   * @description: 获取聊天用户列表
   * @return {*}
   */
  const getRoomList = () => {
    chatLoading.value = true;
    const params: any = {
      uniq_id: imStore.user_im_id,
      keywords: '',
    };
    // const timeRange = formatRoomTime(imStore.otherParams)
    if (Object.keys(imStore.otherParams).length) Object.assign(params, imStore.otherParams)
    return API_getFoldRoomList(params)
      .then((res: any) => {
        if (res === undefined) return;
        if (res.status === 0) {
          allRoomList.value = chatListSort(
            (res.data.room_list || []).filter(item => !item.name.includes('客服')),
          );
          updateUnReadNum();
          handleSliceList(imStore.chatType);
        } else {
          ElMessage.error(res.message);
        }
      })
      .finally(() => {
        chatLoading.value = false;
      });
  };

  /**
   * @description: 更新消息未读数
   * @return {*}
   */
  const updateUnReadNum = () => {
    const numMap: any = {
      all: 0,
      single: 0,
      robot: 0,
      business: 0,
      content: 0,
      group: 0,
    };
    allRoomList.value.map((item: any) => {
      if (item.open_tip === 1) {
        numMap.all += item.total; // 全部
        if (item.type === 'business') {
          numMap.business += item.total; // 服务窗
        } else if (item.type === 'tool' || item.type === 'content' || item.type === 'detail') {
          numMap.robot += item.total; // 助手
        } else if (item.room_type === 0) {
          numMap.single += item.total; // 单聊
        } else if (item.room_type > 1) {
          numMap.group += item.total; // 好友群聊
        }
      }
    });
    imStore.updateUserUnreadNum(numMap);
  };
  const isBusiness = (item: any) =>
    item.type === 'business' || item.type === 'content' || item.type === 'detail';
  const sliceMap: any = {
    // all: () => allRoomList.value,
    all: () => allRoomList.value.filter((item: any) => true),
    single: () =>
      allRoomList.value.filter(
        (item: any) => !isBusiness(item) && item.type !== 'tool' && item.room_type === 0,
      ),
    business: () => allRoomList.value.filter((item: any) => item.type === 'business'),
    content: () => allRoomList.value.filter((item: any) => item.type === 'content'),
    robot: () =>
      allRoomList.value.filter(
        (item: any) => item.type === 'tool' || item.type === 'content' || item.type === 'detail',
      ),
    group: () => allRoomList.value.filter((item: any) => item.room_type > 1 && item.room_type < 4),
  };

  /**
   * @description: 筛选聊天列表
   * @param {string} type
   * @return {*}
   */
  const handleSliceList = (type: string) => {
    roomList.value = sliceMap[type]?.() || [];
  };

  /**
   * @description: 创建房间
   * @param {string} user_id
   * @return {*}
   */
  const createRoom = async (user_id: string) => {
    const params = {
      uniq_id: imStore.user_im_id,
      to_uniq_id: user_id,
    };
    return API_createRoom(params).then((res: any) => {
      if (res.status === 0) {
        return res.data.room_id;
      }
    });
  };

  /**
   * @description: 获取用户信息
   * @param {string} user_id
   * @return {*}
   */
  const getUserInfo = (user_id: string) => {
    const params = {
      uniq_id: imStore.user_im_id,
      user_id: user_id,
    };
    return API_getUserInfo(params).then((res: any) => {
      if (res.status === 0) {
        return res.data;
      }
    });
  };

  /**
   * @description: 将指定项移到第一位
   * @param {string} uniqId
   * @return {*}
   */
  const moveObjectToFront = (data: any) => {
    let index = allRoomList.value.findIndex((item: any) => item.room_id === data.room_id);
    if (index !== -1) {
      let obj = allRoomList.value.splice(index, 1)[0];
      let startIndex =
        data.topping === 1 ? 0 : allRoomList.value.findIndex((item: any) => item.topping !== 1);
      allRoomList.value.splice(startIndex, 0, obj);
      handleSliceList(imStore.chatType);
    }
  };

  /**
   * @description: 如果列表中有此条数据则更新，没有则构造一条新的数据插入
   * @return {*}
   */
  const handleCreateItem = (data: any) => {
    if (!data.room_id) return;
    let current = allRoomList.value.find((item: any) => item.room_id === data.room_id);
    if (current) {
      if (data.is_show === 0) {
        if (data.id === current.last_msg_info.id) {
          // current = Object.assign(current, data);
          current.last_msg_info = data;
          current.msg_type = data.msg_type;
          current.create_time = data.create_time;
          current.is_show = data.is_show;
          if (data.room_type > 0) {
            current.name = data.room_name;
            if (data.hasOwnProperty('is_dissolution')) {
              current.is_dissolution = data.is_dissolution;
            }
            if (data.hasOwnProperty('display_flag')) {
              current.in_channel = data.display_flag;
            }
          }
        }
      } else {
        // current = Object.assign(current, data);
        current.last_msg_info = data;
        current.total = data.unread_total;
        current.msg_type = data.msg_type;
        current.create_time = data.create_time;
        current.is_show = data.is_show;
        current.mention_num = data.mention_num || 0;
        if (data.room_type > 0) {
          current.name = data.room_name;
          if (data.hasOwnProperty('is_dissolution')) {
            current.is_dissolution = data.is_dissolution;
          }
          if (data.hasOwnProperty('display_flag')) {
            current.in_channel = data.display_flag;
          }
        }
      }
    } else {
      // fix: 后端返回的name有问题，前端处理
      if (data.room_type === 0 && data.uniq_id === imStore.user_im_id) {
        data.name = data.to_user?.name || data.name;
        data.uniq_id = data.to_user.uniq_id || data.uniq_id;
        data.type = data.to_user?.type || data.type;
        data.icon = data.to_user?.icon || data.icon;
      }
      const createItem = {
        icon: data.icon || '',
        name: data.room_type > 0 ? data.room_name : data.name || '',
        uniq_id: data.uniq_id || '',
        open_tip: data.open_tip || 1,
        topping: data.topping || 0,
        create_time: data.create_time || '',
        is_show: data.is_show || 1,
        in_channel: data.in_channel || 1,
        self_uniq_id: data.to_user?.uniq_id || '',
        total: data.unread_total || 0,
        mention: data.mention || '',
        mention_num: data.mention_num || 0,
        room_id: data.room_id || 0,
        room_type: data.room_type || 0,
        member_list: data.member_list || [],
        is_manager: data.is_manager || 0,
        is_mention: data.is_mention || 0,
        is_read: data.is_read || 0,
        is_dissolution: data.is_dissolution || 0,
        is_remove: data.is_remove || 0,
        position: data.position || '',
        school: data.school || '',
        type: data.type || '',
        city: data.city || '',
        company: data.company || '',
        in_group: data.in_group || 0,
        last_msg_info: data,
      };
      allRoomList.value.unshift(createItem);
    }
    handleSliceList(imStore.chatType);
    updateUnReadNum();
  };

  /**
   * @description: 如果从其他地方跳转过来，创建新的房间
   * @param {string} user_id
   * @return {*}
   */
  const handleCreateRoomInfo = async (user_id: string) => {
    let room_id = await createRoom(user_id);
    await getRoomList();
    return room_id;
  };

  return {
    roomList,
    chatLoading,
    allRoomList,
    updateUnReadNum,
    handleSliceList,
    moveObjectToFront,
    handleCreateItem,
    getRoomList,
    handleCreateRoomInfo,
  };
};
